#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>
#include <unistd.h>

/*
  WARNING:
  This file originated in CVS in app/7400/.  You probably want to make
  changes there; 'make install' in that directory will automatically
  propogate source code changes to the ts-7200/util/test/ copy of the
  source code.
*/
void *open_mem(int *fd,off_t addr) {
  off_t page;
  unsigned char *start;

  *fd = open("/dev/mem", O_RDWR | O_SYNC);
  if (*fd == -1) {
    perror("open:");
    return 0;
  }
  page = addr & 0xfffff000;
  start = mmap(0, getpagesize(), PROT_READ|PROT_WRITE, MAP_SHARED, *fd, page);
  if (start == MAP_FAILED) {
    perror("mmap:");
    return 0;
  }
  return (void *)(start + (addr & 0xFFF));
}

char *open_dio(int *fd) {
  char *ret;

  ret = open_mem(fd,0x12C00000);
  return ret;
}

int getDR(char *x,int n) {
  if (n < 0 || n > 15) {
    return 0;
  }
  if (n < 8) {
    return (x[0] >> n) & 1;
  } else {
    return (x[1] >> (n-8)) & 1;
  }
}

// 1 = output
void setDR(char *x,int n,int val) {
  if (n < 0 || n > 15) {
    return;
  }
  if (n < 8) {
    x[0] = (x[0] & ~(1 << n)) | (val ? (1<<n) : 0);
  } else {
    n -= 8;
    x[1] = (x[1] & ~(1 << n)) | (val ? (1<<n) : 0);
  }
}

int getD(char *x,int n) {
  printf("."); fflush(stdout);
  usleep(1);
  if (n < 0 || n > 15) {
    return 0;
  }
  if (n < 8) {
    return (x[2] >> n) & 1;
  } else {
    return (x[3] >> (n-8)) & 1;
  }
}

void setD(char *x,int n,int val) {
  if (n < 0 || n > 15) {
    return;
  }
  if (n < 8) {
    //printf("bit %d -> %d, was %02X now=",n,val,x[2]);
    x[2] = (x[2] & ~(1 << n)) | (val ? (1<<n) : 0);
    //printf("%02X\n",x[2]);
  } else {
    n -= 8;
    x[3] = (x[3] & ~(1 << n)) | (val ? (1<<n) : 0);
  }
}

int main() {
  int i,j,fd,errs=0;
  char *X = open_dio(&fd);

  if (!X) {
    return 1;
  }
  for (i=0;i<16;i++) {
    setDR(X,i,0);
    if (getDR(X,i) != 0) {
      printf("Failed to clear DR %d\n",i);
      errs++;
    }
    setDR(X,i,1);
    if (getDR(X,i) != 1) {
      printf("Failed to set DR %d\n",i);
      errs++;
    }
  }
  // Set all even ports as OUTPUT LO, all odd an INPUT
  for (i=0;i<16;i+=2) {
    setDR(X,i,1);
    setD(X,i,0);
  }
  for (i=1;i<16;i+=2) {
    setDR(X,i,0);
  }
  // Test HI even->even+1
  for (i=0;i<16;i+=2) {
    setD(X,i,1);
    if (getD(X,i+1) != 1) {
      printf("%d -> %d (HI) failed [%02X%02X] [%02X%02X]\n",i,i+1,X[1],X[0],X[3],X[2]);
      errs++;
    }
    setD(X,i,0);
  }
  // Set all even ports as HI
  for (i=0;i<16;i+=2) {
    setD(X,i,1);
  }
  // Test LO even->even+1
  for (i=0;i<16;i+=2) {
    setD(X,i,0);
    if (getD(X,i+1) != 0) {
      printf("%d -> %d (LO) failed [%02X%02X] [%02X%02X]\n",i,i+1,X[1],X[0],X[3],X[2]);
      errs++;
    }
    setD(X,i,1);
  }
  // Set all odd ports as OUTPUT LO, all even an INPUT
  for (i=0;i<16;i+=2) {
    setDR(X,i,0);
  }
  for (i=1;i<16;i+=2) {
    setDR(X,i,1);
    setD(X,i,0);
  }
  // Test HI odd-1 <- odd
  for (i=0;i<16;i+=2) {
    setD(X,i+1,1);
    if (getD(X,i) != 1) {
      printf("%d <- %d (HI) failed [%02X%02X] [%02X%02X]\n",i,i+1,X[1],X[0],X[3],X[2]);
      errs++;
    }
    setD(X,i+1,0);
  }
  // Set all odd ports as HI
  for (i=0;i<16;i+=2) {
    setD(X,i+1,1);
  }
  // Test LO odd-1 <- odd
  for (i=0;i<16;i+=2) {
    setD(X,i+1,0);
    if (getD(X,i) != 0) {
      printf("%d <- %d (LO) failed [%02X%02X] [%02X%02X]\n",i,i+1,X[1],X[0],X[3],X[2]);
      errs++;
    }
    setD(X,i+1,1);
  }
  printf("Done with %d errors\n",errs);
  return errs;
}
