#include "ByteArrayFlex.h"

// copyright (c)2006 Technologic Systems
// Author: Michael Schmidt

inline BAFref BAFR(byte *ptr,int len,int max) {
  return (BAFref){max,len,ptr};
}

/*
  1. The way we use this routine to delete is awkward. this function reduces
     the length, but we restore it so we can actually do the delete properly,
     and then we unrestore it.
  2. Perhaps we can implement other kinds of arrays by wrapping them around
     byte arrays and calling byte array functions.
  3. We can implement an even more flexible array that prepends without copy,
     but that will only be a performance optimization.
  4. We can probably call ByteArrayRef__Overwrite instead of the other one.
 */
inline BAFref ByteArrayFlexRef_Bound(BAFref str,int offset,int *len) {
  str.len += *len;
  if (str.len >= str.max) {
    str.len= str.max;
  } else if (str.len < offset) {
    str.len = offset;
  }
  return str;
}

inline BAFref ByteArrayFlexRef_InsertIn(BAFref str,ByteArrayRef str2,int ofs) {
  int len;

  ofs = ((ofs < 0) ? 1 : 0) + ByteArrayRef_Normalize(str.ref,ofs);
  len = str2.len;
  str = ByteArrayFlexRef_Bound(str,ofs,&len);
  
  // move existing string to make room for new string
  ByteArrayRef_OverwriteIn(str.ref,str.ref,ofs+len,ofs,str.len-ofs);

  // copy new string into vacated area
  ByteArrayRef_OverwriteIn(str.ref,str2,ofs,0,len);
  return str;
}

inline BAFref ByteArrayFlexRef_PrependTo(BAFref str,ByteArrayRef newstr) {
  return ByteArrayFlexRef_InsertIn(str,newstr,0);
}

inline BAFref ByteArrayFlexRef_AppendTo(BAFref str,ByteArrayRef newstr) {
  return ByteArrayFlexRef_InsertIn(str,newstr,-1);
}


inline BAFref ByteArrayFlexRef_DeleteFrom(BAFref str,int ofs,int count) {
  int ncount = -count;

  ofs = ByteArrayRef_Normalize(str.ref,ofs);
  str = ByteArrayFlexRef_Bound(str,ofs,&ncount);
  count = -ncount;
  str.len += count;

  // move remainder of existing string over deleted portion
  ByteArrayRef_OverwriteIn(str.ref,str.ref,ofs,ofs+count,str.len-ofs-count);

  // adjust string bounds to shorter length
  str.len -= count;

  return str;
}

inline BAFref ByteArrayRef_Truncate(BAFref str,int newlen) {
  return ByteArrayFlexRef_DeleteFrom(str,newlen,str.len-newlen);
}

BAFref ByteArrayFlexRef_Substring(BAFref str,int offset,int len) {
  BAFref ret;

  // delete end first, so we have less to copy back
  ret = ByteArrayFlexRef_DeleteFrom(str,offset+len,str.len-offset-len);
  // now chop off the front
  return ByteArrayFlexRef_DeleteFrom(ret,0,offset);
}
