#include "mtd.h"

void dump(unsigned char *buf,int len,int offset) {
  int j,k;

  for (j=0;j<len;j++) {
    if (j % 16 == 0) {
      printf("%04X ",j+offset);
    }
    printf("%02X ",buf[j]);
    if (j % 16 == 15) {
      for (k=j-15;k<=j;k++) {
	printf("%c",(buf[k] >= 32 && buf[k] < 127) ? buf[k] : '.');
      }
      printf("\n");
    }
  }
  if (j % 16 != 0) {
    printf("\n");
  }
}

int sciformat(int val,int *prefix,int *fraction) {
  if (val < 1024) {
    *prefix = 1;
    return val;
  }
  if (val < 1024*1024) {
    *prefix = 3;
    *fraction = (val % 1024) / 10;
    return val / 1024;
  }
  if (val < 1024*1024*1024) {
    *prefix = 6;
    *fraction = (val % (1024*1024)) / 10485;
    return val / (1024 * 1024);
  }
  *prefix = 9;
  *fraction = (val % (1024*1024*1024)) / 10737418;
  return val / (1024 * 1024 * 1024);
}

char PWR(int pwr) {
  if (pwr < 3) {
    return 0;
  }
  if (pwr < 6) {
    return 'k';
  }
  if (pwr < 9) {
    return 'M';
  }
  return 'G';
}

int main(int argc,char *argv[]) {
  MTDFILE *f;
  char tmp[256];
  mtd_info_t inf;
  int i,pwr,fract;

  if (argc < 2) {
    printf("Usage: %s <mtdblock>\nwhere <mtdblock> is in /dev/mtdblock/\n",
	   argv[0]);
    return 0;
  }
  sprintf(tmp,"/dev/mtd/%d",atoi(argv[1]));
  f = MTDopen(tmp);
  if (f == NULL) {
    printf("MTDopen(%s) failed: %m\n",tmp);
    return 0;
  }

  printf("type= %X (%d)\n",inf.type,inf.type);
  printf("flags= %X (%d)\n",inf.flags,inf.flags);
  printf("Size=%X (%d)\n",inf.size,inf.size);
  printf("erasesize=%X (%d)\n",inf.erasesize,inf.erasesize);
  printf("oobblock= %X (%d)\n",inf.oobblock,inf.oobblock);
  printf("oobsize= %X (%d)\n",inf.oobsize,inf.oobsize);
  printf("ecctype= %X (%d)\n",inf.ecctype,inf.ecctype);
  printf("eccsize= %X (%d)\n",inf.eccsize,inf.eccsize);

  printf("%d bytes (+%d) per sector\n",f->info.oobblock,f->info.oobsize);
  i = sciformat(f->info.erasesize,&pwr,&fract);
  printf("%d sectors (%d.%d%cB) per block\n",
	 f->info.erasesize/f->info.oobblock,i,fract,PWR(pwr));
  i = sciformat(f->info.size,&pwr,&fract);
  printf("%d blocks (%d.%d%cB) total\n",f->info.size/f->info.erasesize,
	 i,fract,PWR(pwr));
  /*
  for (i=0;i<f->info.erasesize/f->info.oobblock;i++) {
    MTDreadOOB(f,0,i,tmp);
    dump(tmp,f->info.oobsize,i);
  }
  */
}
