
static const unsigned char _nand_ecc_precalc_table[] = {
    0x00, 0x55, 0x56, 0x03, 0x59, 0x0c, 0x0f, 0x5a, 0x5a, 0x0f, 0x0c, 0x59, 
    0x03, 0x56, 0x55, 0x00, 0x65, 0x30, 0x33, 0x66, 0x3c, 0x69, 0x6a, 0x3f, 
    0x3f, 0x6a, 0x69, 0x3c, 0x66, 0x33, 0x30, 0x65, 0x66, 0x33, 0x30, 0x65, 
    0x3f, 0x6a, 0x69, 0x3c, 0x3c, 0x69, 0x6a, 0x3f, 0x65, 0x30, 0x33, 0x66,
    0x03, 0x56, 0x55, 0x00, 0x5a, 0x0f, 0x0c, 0x59, 0x59, 0x0c, 0x0f, 0x5a, 
    0x00, 0x55, 0x56, 0x03, 0x69, 0x3c, 0x3f, 0x6a, 0x30, 0x65, 0x66, 0x33, 
    0x33, 0x66, 0x65, 0x30, 0x6a, 0x3f, 0x3c, 0x69, 0x0c, 0x59, 0x5a, 0x0f, 
    0x55, 0x00, 0x03, 0x56, 0x56, 0x03, 0x00, 0x55, 0x0f, 0x5a, 0x59, 0x0c,
    0x0f, 0x5a, 0x59, 0x0c, 0x56, 0x03, 0x00, 0x55, 0x55, 0x00, 0x03, 0x56, 
    0x0c, 0x59, 0x5a, 0x0f, 0x6a, 0x3f, 0x3c, 0x69, 0x33, 0x66, 0x65, 0x30, 
    0x30, 0x65, 0x66, 0x33, 0x69, 0x3c, 0x3f, 0x6a, 0x6a, 0x3f, 0x3c, 0x69, 
    0x33, 0x66, 0x65, 0x30, 0x30, 0x65, 0x66, 0x33, 0x69, 0x3c, 0x3f, 0x6a,
    0x0f, 0x5a, 0x59, 0x0c, 0x56, 0x03, 0x00, 0x55, 0x55, 0x00, 0x03, 0x56, 
    0x0c, 0x59, 0x5a, 0x0f, 0x0c, 0x59, 0x5a, 0x0f, 0x55, 0x00, 0x03, 0x56, 
    0x56, 0x03, 0x00, 0x55, 0x0f, 0x5a, 0x59, 0x0c, 0x69, 0x3c, 0x3f, 0x6a, 
    0x30, 0x65, 0x66, 0x33, 0x33, 0x66, 0x65, 0x30, 0x6a, 0x3f, 0x3c, 0x69,
    0x03, 0x56, 0x55, 0x00, 0x5a, 0x0f, 0x0c, 0x59, 0x59, 0x0c, 0x0f, 0x5a, 
    0x00, 0x55, 0x56, 0x03, 0x66, 0x33, 0x30, 0x65, 0x3f, 0x6a, 0x69, 0x3c, 
    0x3c, 0x69, 0x6a, 0x3f, 0x65, 0x30, 0x33, 0x66, 0x65, 0x30, 0x33, 0x66, 
    0x3c, 0x69, 0x6a, 0x3f, 0x3f, 0x6a, 0x69, 0x3c, 0x66, 0x33, 0x30, 0x65,
    0x00, 0x55, 0x56, 0x03, 0x59, 0x0c, 0x0f, 0x5a, 0x5a, 0x0f, 0x0c, 0x59, 
    0x03, 0x56, 0x55, 0x00
};

static void 
_nand_trans_result(unsigned char reg2, unsigned char reg3,
                   unsigned char *ecc0, unsigned char *ecc1)
{
    unsigned char a, b, i, tmp1, tmp2;
	
    /* Initialize variables */
    a = b = 0x80;
    tmp1 = tmp2 = 0;
	
    /* Calculate first ECC byte */
    for (i = 0; i < 4; i++) {
        if (reg3 & a)		/* LP15,13,11,9 --> ecc_code[0] */
            tmp1 |= b;
        b >>= 1;
        if (reg2 & a)		/* LP14,12,10,8 --> ecc_code[0] */
            tmp1 |= b;
        b >>= 1;
        a >>= 1;
    }
	
    /* Calculate second ECC byte */
    b = 0x80;
    for (i = 0; i < 4; i++) {
        if (reg3 & a)		/* LP7,5,3,1 --> ecc_code[1] */
            tmp2 |= b;
        b >>= 1;
        if (reg2 & a)		/* LP6,4,2,0 --> ecc_code[1] */
            tmp2 |= b;
        b >>= 1;
        a >>= 1;
    }
	
    /* Store two of the ECC bytes */
    *ecc0 = tmp1;    
    *ecc1 = tmp2;
}

//
// Calculate 3 byte ECC on 256 bytes of data
// copied from redboot
void
_nand_page_ECC(unsigned char *data, unsigned char *ecc0,
               unsigned char *ecc1, unsigned char *ecc2)
{
    unsigned char idx, reg1, reg2, reg3;
    int j;
	
    /* Initialize variables */
    reg1 = reg2 = reg3 = 0;
    *ecc0 = *ecc1 = *ecc2 = 0;
	
    /* Build up column parity */ 
    for(j = 0; j < 256; j++) {
        /* Get CP0 - CP5 from table */
        idx = _nand_ecc_precalc_table[*data++];
        reg1 ^= (idx & 0x3f);
        /* All bit XOR = 1 ? */
        if (idx & 0x40) {
            reg3 ^= (unsigned char) j;
            reg2 ^= ~((unsigned char) j);
        }
    }
	
    /* Create non-inverted ECC code from line parity */
    _nand_trans_result(reg2, reg3, ecc0, ecc1);
	
    /* Calculate final ECC code */
    *ecc0 = ~*ecc0;
    *ecc1 = ~*ecc1;
    *ecc2 = ((~reg1) << 2) | 0x03;
}

//----------------------------------------------------------------------------
#define FLAG_ERROR(x,args...) // fprintf(stderr,"%s:%d:" x "\n",__FILE__,__LINE__,args)

#define OOB_STATUS_EMPTY 0
#define OOB_STATUS_INUSE 1
#define OOB_STATUS_BAD -1

inline int sectorOffset(int block,int sector) {
  return mtd_info.erasesize * block +  mtd_info.oobblock * sector;
}

int eraseBlock(FILE *dst,int block) {
  int ret;
  erase_info_t erase;

  erase.start = block * mtd_info.erasesize;
  erase.length = mtd_info.erasesize;

  ret = ioctl(fileno(dst), MEMERASE, &erase);
  if (ret == -1) {
    FLAG_ERROR("eraseBlock:ioctl returned %d (%m)",ret);
    return 0;
  }
  if (ferror(dst)) {
    FLAG_ERROR("eraseBlock:ferror=%d after ioctl (%m)",ferror(dst));
    return 0;
  }
  return 1;
}

void markBlockBad(FILE *dst,int block) {
  struct mtd_oob_buf oob_buf;
  unsigned char oob[mtd_info.oobsize];

  //  fprintf(stderr,"***************** Marking block %d bad\n",block);
  syslog(LOG_NOTICE,"RedBoot partition block %d bad\n",block);
  if (!eraseBlock(dst,block)) {
    syslog(LOG_NOTICE,"unable to mark block %d bad (block won't erase)\n",
	block);
    fprintf(stderr,"Unable to deal with bad flash block %d, exiting...\n",
	block);
    exit(3);
  }
  oob_buf.length = mtd_info.oobsize;
  oob_buf.ptr = oob;
  oob_buf.start = mtd_info.erasesize * block;

  assert(ioctl(fileno(dst), MEMREADOOB, &oob_buf) != -1);
  assert(!ferror(dst));
  oob[0] = 0;    // mark sector as throughly completely bad
  oob[1] = 0;    // mark sector as throughly completely bad
  ioctl(fileno(dst), MEMWRITEOOB, &oob_buf);
  oob[0] = 0xFF; // in case read fails
  oob[1] = 0xFF; // in case read fails
  ioctl(fileno(dst), MEMREADOOB, &oob_buf);
  if (oob[0] == 0xFF || oob[1] == 0xFF) {
    syslog(LOG_NOTICE,"unable to mark block %d bad (OOB write failed)\n",block);
    fprintf(stderr,"Unable to deal with bad flash block %d, exiting...\n",
	block);
    exit(3);
  }
}

int seekTo(FILE *f,int block,int sector) {
  int ret;
  //so = sectorOffset(block,sector); //we never use this

 retry:
  ret = fseek(f,sectorOffset(block,sector),SEEK_SET);
  if (ret != 0) {
    FLAG_ERROR("fseek returned %d (%m)",ret);
    exit(3);
    goto retry;
    return 0;
  }
  return 1;
}

int getOOB(FILE *f,int block,int sector,char *oob) {
  struct mtd_oob_buf oob_buf;
  int ret;

  if (!seekTo(f,block,sector)) {
    FLAG_ERROR("failed to seek to block %d, sector %d",block,sector);
    return 0;
  }

  oob_buf.length = mtd_info.oobsize;
  oob_buf.ptr = oob;
  oob_buf.start = sectorOffset(block,sector);;

  ret = ioctl(fileno(f), MEMREADOOB, &oob_buf);
  if (ret == -1) {
    FLAG_ERROR("getOOB:ioctl returned %d (%m)",ret);
    return 0;
  }
  if (ferror(f)) {
    FLAG_ERROR("getOOB:ferror=%d after ioctl (%m)",ferror(f));
    return 0;
  }
  return 1;
}

int getSectorStatus(FILE *f,int block,int sector) {
  unsigned char oob[mtd_info.oobsize];
  int i;

  if (!getOOB(f,block,sector,oob)) {
    FLAG_ERROR("getOOB failed", 0);
    return OOB_STATUS_BAD;
  }
  /* WHY is this commented out?
  if (oob[0] != 0xFF) {
    return OOB_STATUS_BAD;
  }
  */
  for (i=0;i<mtd_info.oobsize;i++) {
    if (oob[i] != 0xFF) {
      return OOB_STATUS_INUSE;
    }
  }
  return OOB_STATUS_EMPTY;
}

void dumpBuffer(unsigned char *buf,int len) {
  int j,k;

  printf("Buffer at %p\n",buf);
  for (j=0;j<len;j++) {
    if (j % 16 == 0) {
      printf("%04X ",j);
    }
    printf("%02X ",buf[j]);
    if (j % 16 == 15) {
      for (k=j-15;k<=j;k++) {
	printf("%c",(buf[k] >= 32 && buf[k] < 127) ? buf[k] : '.');
      }
      printf("\n");
    }
  }
  printf("\n");
}

int readSector(FILE *f,char *buffer) {
  int ret;
  //int x = (ftell(f) == 0);

  memset(buffer,0,mtd_info.oobblock);
  ret = fread(buffer,1,mtd_info.oobblock,f);
  if (ret != mtd_info.oobblock) {
    FLAG_ERROR("readSector:fread read only %d bytes of sector (%m)",ret);
    return 0;
  }
  /*
  if (x == 1) {
    printf("readSector@0 on %p\n",f);
    dumpBuffer(buffer,256);
  }
  */
  if (ferror(f)) {
    FLAG_ERROR("readSector:ferror=%d after fread (%m)",ferror(f));
    return 0;
  }
  return 1;
}

int writeSector(FILE *f,char *buffer) {
  int ret;

  ret = fwrite(buffer,mtd_info.oobblock,1,f);
  if (ret != 1) {
    FLAG_ERROR("writeSector:fwrite wrote %d (of 1) blocks (%m)",ret);
    return 0;
  }
  if (ferror(f)) {
    FLAG_ERROR("writeSector:ferror=%d after fwrite (%m)",ferror(f));
    return 0;
  }
  return 1;
}

int verifySectorAppearsEmpty(int block,int sector,FILE *f) {
  int i;
  unsigned char buf[mtd_info.oobblock];

  if (!seekTo(f,block,sector)) {
    FLAG_ERROR("failed to seek to block %d, sector %d",block,sector);
    return 0;
  }
  if (!readSector(f,buf)) {
    FLAG_ERROR("readSector failed",0);
    return 0;
  }
  for (i=0;i<mtd_info.oobblock;i++) {
    if (buf[i] != 0xFF) {

      FLAG_ERROR("block %d, sector %d, byte %d=0x%02X",block,sector,i,buf[i]);
      /*
      if (sector == 0) {
	printf("At offset %ld\n",ftell(f)-2048);
	dumpBuffer(buf,128);
      }
      */
      return 0;
    }
  }  
  if (getSectorStatus(f,block,sector) != OOB_STATUS_EMPTY) {
    FLAG_ERROR("block %d, sector %d, OOB data not empty",block,sector);
    return 0;
  }
  if (!seekTo(f,block,sector)) {
    return 0;
  }
  return 1;
}

int writeAndVerifySector(char *buffer,int block,int sector, FILE *f) {
  char buf2[mtd_info.oobblock];

  if (!seekTo(f,block,sector)) {
    FLAG_ERROR("failed to seek to block %d, sector %d",block,sector);
    return 0;
  }
  if (!writeSector(f,buffer)) {
    FLAG_ERROR("writeSector failed",0);
    return 0;
  }
  if (!seekTo(f,block,sector)) {
    FLAG_ERROR("failed to seek to block %d, sector %d",block,sector);
    return 0;
  }
  if (!readSector(f,buf2)) {
    FLAG_ERROR("readSector failed",0);
    return 0;
  }
  if (memcmp(buffer,buf2,mtd_info.oobblock)) {
    FLAG_ERROR("block %d, sector %d verify failed",block,sector);
    return 0;
  }
  if (getSectorStatus(f,block,sector) == OOB_STATUS_BAD) {
    FLAG_ERROR("block %d, sector %d MURDERED!!!",block,sector);
    exit(3); // murder warrant immediate termination!
  }
  return 1;
}

int writeSectorBufferToFlash(char *buffer,int block,int sector,FILE *f) {

  if (!seekTo(f,block,sector)) {
    FLAG_ERROR("failed to seek to block %d, sector %d",block,sector);
    return 0;
  }
  if (!verifySectorAppearsEmpty(block,sector,f)) {
    FLAG_ERROR("block %d, sector %d not empty",block,sector);
    return 0;
  }
  if (!writeAndVerifySector(buffer,block,sector,f)) {
    FLAG_ERROR("write/verify block %d, sector %d failed",block,sector);
    return 0;
  }
  return 1;
}


int readSectorAndWriteToFlash(FILE *src,FILE *dst,int block,int sector) {
  char buffer[mtd_info.oobblock];

  if (!readSector(src,buffer)) {
    FLAG_ERROR("readSector(src) failed",0);
    return 0;
  }
  crc = crc32_accumulate(crc, buffer, mtd_info.oobblock);	
  if (!writeSectorBufferToFlash(buffer,block,sector,dst)) {
    FLAG_ERROR("writeSectorBufferToFlash failed at block %d, sector %d",
	block,sector);
    return 0;
  }
  return 1;
}

int readBlockAndWriteToFlash(FILE *src,FILE *dst,int block) {
  int i,sectors;

  sectors = mtd_info.erasesize / mtd_info.oobblock;

  if (!eraseBlock(dst,block)) {
    FLAG_ERROR("unable to erase block %d",block);
    return 0;
  }
  for (i=0;i<sectors;i++) {
    if (!readSectorAndWriteToFlash(src,dst,block,i)) {
      if (feof(src)) {
	//	fprintf(stderr,"EOF reading source file\n");
      } else {
	FLAG_ERROR("readSectorAndWriteToFlash failed at sector %d, marking 
		block %d bad",i,block);
	markBlockBad(dst,block);
      }
      return 0;
    }
  }
  return 1;
}
//----------------------------------------------------------------------------


// Copied from redboot (fis.h)
#define CYG_ADDRESS unsigned
#define FIS_IMAGE_DESC_SIZE_UNPADDED \
  (16 + 4 * sizeof(unsigned long) + 3 * sizeof(CYG_ADDRESS))
#define CYGNUM_REDBOOT_FIS_DIRECTORY_ENTRY_SIZE 64

char fisbuf[4096];
struct fis_image_desc {
    unsigned char name[16];      // Null terminated name
    CYG_ADDRESS   flash_base;    // Address within FLASH of image
    CYG_ADDRESS   mem_base;      // Address in memory where it executes
    unsigned long size;          // Length of image
    CYG_ADDRESS   entry_point;   // Execution entry point
    unsigned long data_length;   // Length of actual data
    unsigned char _pad[CYGNUM_REDBOOT_FIS_DIRECTORY_ENTRY_SIZE-FIS_IMAGE_DESC_SIZE_UNPADDED];
    unsigned long desc_cksum;    // Checksum over image descriptor
    unsigned long file_cksum;    // Checksum over image data
} *fid = (struct fis_image_desc *)fisbuf;
char fconfig[4096] = {
0x00,0x10,0x00,0x00,0xCE,0xFA,0xAD,0x0B,0x01,0x0C,0x01,0x00,0x62,0x6F,0x6F,0x74,
0x5F,0x73,0x63,0x72,0x69,0x70,0x74,0x00,0x01,0x00,0x00,0x00,0x04,0x11,0x01,0x0C,
0x62,0x6F,0x6F,0x74,0x5F,0x73,0x63,0x72,0x69,0x70,0x74,0x5F,0x64,0x61,0x74,0x61,
0x00,0x62,0x6F,0x6F,0x74,0x5F,0x73,0x63,0x72,0x69,0x70,0x74,0x00,0x66,0x69,0x73,
0x20,0x6C,0x6F,0x61,0x64,0x20,0x76,0x6D,0x6C,0x69,0x6E,0x75,0x78,0x0A,0x65,0x78,
0x65,0x63,0x20,0x2D,0x63,0x20,0x22,0x63,0x6F,0x6E,0x73,0x6F,0x6C,0x65,0x3D,0x74,
0x74,0x79,0x41,0x4D,0x30,0x2C,0x31,0x31,0x35,0x32,0x30,0x30,0x20,0x72,0x6F,0x6F,
0x74,0x3D,0x2F,0x64,0x65,0x76,0x2F,0x6D,0x74,0x64,0x62,0x6C,0x6F,0x63,0x6B,0x31,
0x22,0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x14,0x01,
0x0C,0x62,0x6F,0x6F,0x74,0x5F,0x73,0x63,0x72,0x69,0x70,0x74,0x5F,0x74,0x69,0x6D,
0x65,0x6F,0x75,0x74,0x00,0x62,0x6F,0x6F,0x74,0x5F,0x73,0x63,0x72,0x69,0x70,0x74,
0x00,0x01,0x00,0x00,0x00,0x01,0x06,0x01,0x00,0x62,0x6F,0x6F,0x74,0x70,0x00,0x00,
0x00,0x00,0x00,0x05,0x14,0x00,0x06,0x62,0x6F,0x6F,0x74,0x70,0x5F,0x6D,0x79,0x5F,
0x67,0x61,0x74,0x65,0x77,0x61,0x79,0x5F,0x69,0x70,0x00,0x62,0x6F,0x6F,0x74,0x70,
0x00,0xC0,0xA8,0x00,0x01,0x05,0x0C,0x00,0x06,0x62,0x6F,0x6F,0x74,0x70,0x5F,0x6D,
0x79,0x5F,0x69,0x70,0x00,0x62,0x6F,0x6F,0x74,0x70,0x00,0xC0,0xA8,0x00,0x32,0x05,
0x11,0x00,0x06,0x62,0x6F,0x6F,0x74,0x70,0x5F,0x6D,0x79,0x5F,0x69,0x70,0x5F,0x6D,
0x61,0x73,0x6B,0x00,0x62,0x6F,0x6F,0x74,0x70,0x00,0xFF,0xFF,0xFF,0x00,0x05,0x10,
0x01,0x00,0x62,0x6F,0x6F,0x74,0x70,0x5F,0x73,0x65,0x72,0x76,0x65,0x72,0x5F,0x69,
0x70,0x00,0xC0,0xA8,0x00,0x01,0x06,0x0B,0x01,0x00,0x65,0x70,0x39,0x33,0x78,0x78,
0x5F,0x65,0x73,0x61,0x00,0x00,0xD0,0x69,0x4F,0x53,0x73,0x00,0x00,0x02,0x09,0x01,
0x00,0x67,0x64,0x62,0x5F,0x70,0x6F,0x72,0x74,0x00,0x28,0x23,0x00,0x00,0x01,0x13,
0x01,0x00,0x69,0x6E,0x66,0x6F,0x5F,0x63,0x6F,0x6E,0x73,0x6F,0x6C,0x65,0x5F,0x66,
0x6F,0x72,0x63,0x65,0x00,0x00,0x00,0x00,0x00,0x02,0x14,0x01,0x13,0x69,0x6E,0x66,
0x6F,0x5F,0x63,0x6F,0x6E,0x73,0x6F,0x6C,0x65,0x5F,0x6E,0x75,0x6D,0x62,0x65,0x72,
0x00,0x69,0x6E,0x66,0x6F,0x5F,0x63,0x6F,0x6E,0x73,0x6F,0x6C,0x65,0x5F,0x66,0x6F,
0x72,0x63,0x65,0x00,0x00,0x00,0x00,0x00,0x01,0x0A,0x01,0x00,0x6E,0x65,0x74,0x5F,
0x64,0x65,0x62,0x75,0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xAD,0xDE,0xAD,0xDE,0x2B,0x71,0x1F,0x45
};


unsigned long fis_checksum(struct fis_image_desc *fi) {
  return 0; // haven't found any evidence this is actually used!
}

void add_bad_block(int block) {
  int i;

  for (i=0;i<64;i++) {
    if (fid[i].name[0] == 0xFF) {
      break;
    }
  }
  assert(i<64);
  strcpy(fid[i].name,"(bad block)");
  fid[i].mem_base = fid[i].flash_base = flashBase + block * mtd_info.erasesize;
  fid[i].size = mtd_info.erasesize;
  fid[i].entry_point = -1;
  fid[i].data_length = 0;
  for (i=0;i<CYGNUM_REDBOOT_FIS_DIRECTORY_ENTRY_SIZE - 
	FIS_IMAGE_DESC_SIZE_UNPADDED;i++) {
    fid[i]._pad[i] = 0;
  }
  fid[i].desc_cksum = 0; // fis_checksum(&fid[i]);
  fid[i].file_cksum = 0;
}

void setFconfig(unsigned char *macaddr, unsigned char *bscript) {
  struct fconfig *f = (struct fconfig *)fconfig;
  
  f->cksum = crc32(fconfig,sizeof(struct fconfig)-sizeof(unsigned long));
}

void getFconfig(int i,int j,FILE *src,unsigned char *macaddr, 
		unsigned char *bscript) {
  struct fconfig *f = (struct fconfig *)fconfig;
  char *dp;


  if(tsmodel() == TS7300)
  {
	ee_init();
	ee_read(0x7DC, buf);
	ee_read(0x7E0, buf+4);
	memcpy(srcmac, buf, 6);
  }
  else
  {
  	dp = flash_lookup_config(f, "ep93xx_esa");
  	if (dp) {
    		// modify the fconfig data
    		if (macaddr) {
      			memcpy(srcmac, FCONFIG_OBJECT_VALUE(dp), 6);
      			memcpy(FCONFIG_OBJECT_VALUE(dp), macaddr, 6);
    		}
    		dp = flash_lookup_config(f, "boot_script_data");
    		assert(dp);
    		if (bscript) {
      			int len;
      
      			len = fconfig_lengths[FCONFIG_OBJECT_TYPE(dp)];
      			memcpy(srcbscript, FCONFIG_OBJECT_VALUE(dp),
	     			len);
      			bzero(FCONFIG_OBJECT_VALUE(dp), len);
      			memcpy(FCONFIG_OBJECT_VALUE(dp), bscript, 
	     		strlen(bscript) + 1);
    		}
   		f->cksum = crc32(fconfig,
		  sizeof(struct fconfig)-sizeof(unsigned long));
  	}
  }
}

int getMTDInfo(FILE *ff,mtd_info_t *inf) {
  if (ioctl(fileno(ff), MEMGETINFO, inf) == -1) {
    perror("ioctl:MEMGETINFO");
    return 0;
  }
  return 1;
}

int writeTwoSectorBufferToFlash(char *buffer,int block,int sector,FILE *f) {
  if (!writeSectorBufferToFlash(buffer,block,sector,f)) {
    FLAG_ERROR("error writing first sector",0);
    return 0;
  }
  if (!writeSectorBufferToFlash(buffer+mtd_info.oobblock,block,sector+1,f)) {
    FLAG_ERROR("error writing second sector",0);
    return 0;
  }
  return 1;
}

int writeBufferToFlash(char *buffer,int block,int sector,int bytes,FILE *f) {
  int i,sectors;

  // first, round buffer size up to nearest sector boundary, if needed
  if (bytes % mtd_info.oobblock) {
    fprintf(stderr,"Warning: writeBufferToFlash(%d) wanted multiple of %d, 
	padding\n",bytes,mtd_info.oobblock);
    bytes += mtd_info.oobblock - bytes % mtd_info.oobblock;
  }
  sectors = bytes / mtd_info.oobblock;
  for (i=0;i<sectors;i++) {
    if (!writeSectorBufferToFlash(buffer,block,sector+i,f)) {
      FLAG_ERROR("error writing block %d, sector %d",block,sector+i);
      return 0;
    }
    buffer += mtd_info.oobblock;
  }
  return 1;
}

void set_dir_entry(int n,char *str,unsigned adrs,unsigned len,unsigned entry) {
  strcpy(fid[n].name,str);
  fid[n].flash_base = adrs;
  if (entry) {
    fid[n].mem_base = entry;
  } else {
    fid[n].mem_base = adrs;
  }
  fid[n].size = len; // round up to nearest block
  fid[n].entry_point = entry;
  fid[n].data_length = len;
}
